/**********************************************************************
 *  Launchpad demprogram: ADC_simple_ref1
 *
 * Egyszer mrs ADC-vel, az 1,5 V-os bels referencia hasznlatval.
 * Az A5 (P1.5 lb) analg bemenetre 0 - 1,5 V kztti jelet vigynk.
 * Emellett megmrjk a bels hmr jelt is (Chan 10).
 *
 * A mrs eredmnyeit egyirny szoftveres UART kezelssel (csak adatklds) 
 * kiratjuk. Paramterek: 9600 baud, 8 adatbit, nincs paritsbit, 2 stopbit
 * 
 * A gyrilag kalibrlt 1 MHz-es DCO rajelet hasznljuk: 
 * ACLK = n/a, MCLK = SMCLK = DCO 1 MHz
 * 
 * Hardver kvetelmnyek:
 *  - Launchpad MSP430G2xxx mikrovezrlvel
 *  - Az jabb kiads (v1.5) krtyn az RXD,TXD tktseket 
 *    SW llsba kell helyezni (a tbbi tktssel prhuzamosan)
 *
 *
 *       MSP430G2231, MSP430G2452 vagy MSP430G2553
 *             -----------------
 *         /|\|              XIN|-
 *          | |                 |
 *          --|RST          XOUT|-
 *            |                 |
 *     TxD <--|P1.1         P1.5|<-- 0-1,5 V
 *
 * 
 *   
 *  I. Cserny
 *  MTA ATOMKI, Debrecen
 *  Nov 2012
 *  Fejleszti krnyezet: IAR Embedded Workbench for MSP430 v5.51
 **********************************************************************/
#include "io430.h"
#include "stdint.h"

#define TXD       BIT1                 // TXD a P1.1 lbon

/*-------------------------------------------------------------
 * Egyszeri mrs egy ADC csatornban, 1,5 V a bels referencia
 *-------------------------------------------------------------
 * chan - csatornavlaszt bitek (a csatorna sorszma << 12)
 */
uint16_t ADC_single_meas_REF1_5V(uint16_t chan) {
  ADC10CTL0 &= ~ENC;                   //Az ADC letiltsa jrakonfigurlshoz
  ADC10CTL0 = ADC10SHT_3               //mintavtel: 64 rats
             | ADC10ON                 //Az ADC bekapcsolsa
             | SREF_1                  //VR+ = VREF+ s VR- = AVSS
             | REFON;                  //Bels referencia bekapcsolsa
                                       //1,5 V-os referencia kivlasztsa
  ADC10CTL1 = ADC10SSEL_0 + chan;      //csatorna = 'chan', ADC10OSC az rajel
  ADC10CTL0 |= ENC + ADC10SC;          //Konverzi engedlyezse s indtsa
  while (ADC10CTL1 & BUSY);            //Vrakozs a konverzi vgre
  return ADC10MEM;                     //Visszatrs a konverzi eredmnyvel 
}


/**------------------------------------------------------------
 *   Ksleltet eljrs (1 - 65535 ms)
 *-------------------------------------------------------------
 * delay - a ksleltets ms egysgben megadva
 */
void delay_ms(uint16_t delay) {
  uint16_t i;
  for(i=0; i<delay; i++) {        //"delay"-szer ismteljk
    __delay_cycles(1000);         //1 ms ksleltets
  }
}

/**------------------------------------------------------------
 *   Egy karakter kikldse a soros portra
 *   SW_UART: 9600 bit/s, 8, N, 2 formtum
 *   DCO = 1 MHz (Bitid = 104.167 usec)
 *-------------------------------------------------------------
 * c - a kikldeni kvnt karakter kdja
 */
void sw_uart_putc(char c) {
  uint8_t i;
  uint16_t TXData;                //Adatregiszter (11 bites)
  TXData = (uint16_t)c | 0x300;   //2 Stop bit (mark)
  TXData = TXData << 1;           //Start bit (space)
  for(i=0; i<11; i++) {           //11 bitet kldnk ki
    if(TXData & 0x0001) {         //Soron kvetkez bit vizsglata
      P1OUT |= TXD;               //Ha '1'
    } else {
      P1OUT &= ~TXD;              //Ha '0'
    }
    TXData = TXData >> 1;         //Adatregiszter lptets jobbra
    __delay_cycles(89);           //<== Itt kell hangolni!
  }
  P1OUT |= TXD;                   //Az alaphelyzet: mark
}



/**------------------------------------------------------------
 *  Karakterfzr kirsa a soros portra
 *-------------------------------------------------------------
 * p_str - karakterfzr mutat (nullval lezrt stringre mutat)
 */
void sw_uart_puts(char* p_str) {
  char c;
  while ((c=*p_str)) {
      sw_uart_putc(c);
      p_str++;
  }
}

/**------------------------------------------------------------
 * Eljel nlkli egsz szm kirsa hexadecimlis formban
 *-------------------------------------------------------------
 * t - a kirand szm (uint16_t tpus)
 */
void sw_uart_out4hex(uint16_t t)
{
  char c;
  c=(char)((t>>12) & 0x0F);
  if (c>9) c+=7;
  sw_uart_putc(c+'0');
  c=(char)((t>>8) & 0x0F);
  if (c>9) c+=7;
  sw_uart_putc(c+'0');
  c=(char)((t>>4) & 0x0F);
  if (c>9) c+=7;
  sw_uart_putc(c+'0');
  c=(char)(t & 0x0F);
  if (c>9) c+=7;
  sw_uart_putc(c+'0');
}

/**------------------------------------------------------------
 * Decimlis kirats adott szm tizedesjegyre.
 *-------------------------------------------------------------
 * data - a kirand szm (eljelesen)
 * ndigits - a kirand tizedesek szma
 */
void sw_uart_outdec(int32_t data, uint8_t ndigits) {
  static char sign, s[12];
  int8_t i;
  i=0; sign=' ';
  if(data<0) {                         //Az eljel meghatrozsa
    sign='-'; 
    data = -data;
  }
    do {
      s[i]=data%10 + '0';              //A szmjegyek meghatrozsa
      data=data/10;                    //htulrl visszafel
      i++;
      if(i==ndigits) {s[i]='.'; i++;}  //A tizedespont beszrsa
    } while(data>0 || i<(ndigits+2));
    sw_uart_putc(sign);                //Az eljel kirsa
    do {
      sw_uart_putc(s[--i]);            //A szmjegyek kirsa
    } while(i);
}

void main(void) {
uint16_t data;
int32_t temp;
  WDTCTL = WDTPW + WDTHOLD;            //Letiltjuk a watchdog idztt
  DCOCTL = CALDCO_1MHZ;                // DCO belltsa a gyrilag kalibrlt 
  BCSCTL1 = CALBC1_1MHZ;               // 1 MHz-es frekvencira  
  P1DIR |= TXD;                        //TXD legyen digitlis kimenet
  P1OUT |= TXD;                        //TXD alaphelyzete: mark
//--- P1.3 bels felhzs engedlyezse -----------------------  
  P1DIR &= ~BIT3;                      //P1.3 legyen digitlis bemenet
  P1OUT |= BIT3;                       //Felfel hzzuk, nem lefel  
  P1REN |= BIT3;                       //Bels felhzs engedlyezse   
//--- Analg csatornk engedlyezse --------------------------
  ADC10AE0 |= BIT5;                    //P1.5 legyen analg bemenet
  while(1) {
    delay_ms(1000);
    data = ADC_single_meas_REF1_5V(INCH_5);
    sw_uart_puts("chan 5 = ");
    sw_uart_out4hex(data);
    temp = (int32_t)data*1500L/1023;   //A mV-okban mrt feszltsg
    sw_uart_outdec(temp,3);            //kirs 3 tizedesre
    sw_uart_puts("V chan 10 = ");      
    data = ADC_single_meas_REF1_5V(INCH_10);
    sw_uart_out4hex(data);   
    temp = ((int32_t)data*270687L - 182023932L) >> 16;  
    sw_uart_puts(" temp = "); 
    sw_uart_outdec(temp,1);       
    sw_uart_puts(" C\r\n");   
    delay_ms(1000);    
  }
}
