/************************************************************************************
 *  Launchpad demprogram: uart_soft
 *
 * Szveg kiratsa soros kapcsolaton keresztl, egyirny szoftveres UART kezels. 
 * Paramterek: 2400 baud, 8 adatbit, nincs paritsbit, 1 stopbit 
 * A gyrilag kalibrlt 1 MHz-es DCO rajelet hasznljuk: 
 * ACLK = n/a, MCLK = SMCLK = DCO 1 MHz
 * 
 * Hardver kvetelmnyek:
 *  - Launchpad MSP430G2xxx mikrovezrlvel
 *  - Az jabb kiads (v1.5) krtyn az RXD,TXD tktseket 
 *    SW llsba kell helyezni (a tbbi tktssel prhuzamosan)
 *
 *                MSP430G2xxx
 *             -----------------
 *         /|\|              XIN|-
 *          | |                 |
 *          --|RST          XOUT|-
 *            |                 |
 *     TxD <--|P1.1             |
 *
 * 
 *  I. Cserny
 *  MTA ATOMKI, Debrecen
 *  June 2012
 *  Fejleszti krnyezet: IAR Embedded Workbench for MSP430 v5.30.1 Kickstart Edition
 ************************************************************************************/
#include "io430.h"
#include "stdint.h"

#define TXD       BIT1            // TXD a P1.1 lbon


/**----------------------------------------------
 *   Ksleltet eljrs (1 - 65535 ms)
 *-----------------------------------------------
 * delay - a ksleltets ms egysgben megadva
 */
void delay_ms(uint16_t delay) {
  uint16_t i;
  for(i=0; i<delay; i++) {
    __delay_cycles(1000);         //1 ms ksleltets
  }
}

/**----------------------------------------------
 *   Egy karakter kikldse a soros portra
 *   SW_UART: 9600 bit/s, 8, N, 1 formtum
 *   DCO = 1 MHz (Bitid = 104,17 usec)
 *-----------------------------------------------
 * c - a kikldeni kvnt karakter kdja
 */
void sw_uart_putc(char c) {
  uint8_t i;
  uint16_t TXData;
  TXData = (uint16_t)c | 0x100;   //Stop bit (mark)
  TXData = TXData << 1;           //Start bit (space)
  for(i=0; i<10; i++) {
    if(TXData & 0x0001) {         //Soron kvetkez bit vizsglata
      P1OUT |= TXD;               //Ha '1'
    } else {
      P1OUT &= ~TXD;              //Ha '0'
    }
    TXData = TXData >> 1;         //Adatregiszter lptets jobbra
    __delay_cycles(89);          //<== Itt kell hangolni!
  }
  P1OUT |= TXD;                   //Az alaphelyzet: mark
}

/**----------------------------------------------
 *  Karakterfzr kirsa a soros portra
 *-----------------------------------------------
 * p_str - karakterfzr mutat (nullval lezrt stringre mutat)
 */
void sw_uart_puts(char* p_str) {
  char c;
  while ((c=*p_str)) {            //Amg a lezr nullt el nem rjk
      sw_uart_putc(c);            //egy karakter kirs
      p_str++;                    //mutat lptetse 
  }
}

void main(void) {
  WDTCTL = WDTPW + WDTHOLD;       //Letiltjuk a watchdog idztt
  DCOCTL = CALDCO_1MHZ;           // DCO belltsa a gyrilag kalibrlt 
  BCSCTL1 = CALBC1_1MHZ;          // 1 MHz-es frekvencira  
//--- A TXD kimenet belltsa ----------------------------------  
  P1DIR |= TXD;                   //TXD legyen digitlis kimenet
  P1OUT |= TXD;                   //TXD alaphelyzete: mark
//---  P1.3 bemenet legyen, bels felhzst engedlyezzk ------
  P1DIR &= ~BIT3;                 //P1.3 legyen digitlis bemenet
  P1OUT |= BIT3;                  //Felfel hzzuk, nem lefel  
  P1REN |= BIT3;                  //Bels felhzs engedlyezse    
  while(1) {
    delay_ms(1000);               //1 msodperc vrakozs 
    sw_uart_puts("Hello world!\r\n");  //Egy sor kirsa
  }
}

